@@ -34,6 +34,23 @@ extern "C"
 {
 #endif
 
+/*
+ * Version handling.
+ */
+extern const unsigned int fusionsound_major_version;
+extern const unsigned int fusionsound_minor_version;
+extern const unsigned int fusionsound_micro_version;
+extern const unsigned int fusionsound_binary_age;
+extern const unsigned int fusionsound_interface_age;
+
+/*
+ * Check for a certain FusionSound version.
+ * In case of an error a message is returned describing the mismatch.
+ */
+const char * FusionSoundCheckVersion( unsigned int required_major,
+                                      unsigned int required_minor,
+                                      unsigned int required_micro );
+
 /*
  * Main FusionSound interface.
  */
@@ -92,6 +109,14 @@ DFBResult FusionSoundError(
                             const char *msg,    /* optional message */
                             DFBResult   result  /* result code to interpret */
                           );
+                          
+/*
+ * Behaves like FusionSoundError, but shuts down the calling application.
+ */
+DFBResult FusionSoundErrorFatal(
+                            const char *msg,    /* optional message */
+                            DFBResult   result  /* result code to interpret */
+                          );
 
 /*
  * Returns a string describing 'result'.
@@ -99,22 +124,73 @@ DFBResult FusionSoundError(
 const char *FusionSoundErrorString(
                                     DFBResult result
                                   );
+                                  
+/*
+ * Retrieves information about supported command-line flags in the
+ * form of a user-readable string formatted suitable to be printed
+ * as usage information.
+ */
+const char *FusionSoundUsageString( void );
+
+/*
+ * @internal
+ *
+ * Encodes format constants in the following way (bit 31 - 0):
+ *
+ * 0000:0000 | 0000:0dcc | cccc:cbbb | bbbb:aaaa
+ *
+ * a) sampleformat index<br>
+ * b) total bits per sample<br>
+ * c) effective sound bits per sample (i.e. depth)<br>
+ * d) signed sample format
+ */
+#define FS_SAMPLEFORMAT( index, bits, depth, sgned ) \
+     ( ((index & 0x0F)      ) |                      \
+       ((bits  & 0x7F) <<  4) |                      \
+       ((depth & 0x7F) << 11) |                      \
+       ((sgned & 0x01) << 18) )
 
 /*
  * The sample format is the way of storing audible information.
  *
- * Data is always stored in <b>native endian</b>. This keeps the library and
+ * 8, 16 and 32 bit samples are always stored in <b>native endian</b>, while
+ * 24 bit samples are stored in <b>little endian</b>. This keeps the library and
  * applications simple and clean. Always access sample buffers like arrays of
- * 8 bit or 16 bit integers depending on the sample format, unless data is
+ * 8, 16 or 32 bit integers depending on the sample format, unless data is
  * written with endianness being taken care of. This does not excuse from endian
  * conversion that might be necessary when reading data from files.
  */
 typedef enum {
-     FSSF_UNKNOWN        = 0x00000000,      /* Unknown or invalid format. */
-     FSSF_S16            = 0x00000001,      /* Signed 16 bit (linear). */
-     FSSF_U8             = 0x00000002       /* Unsigned 8 bit (linear). */
+     FSSF_UNKNOWN        = 0x00000000, /* Unknown or invalid format. */
+     
+     /* Unsigned 8 bit. */ 
+     FSSF_U8             = FS_SAMPLEFORMAT( 0,  8,  8, 0 ),
+     
+     /* Signed 16 bit (native endian). */
+     FSSF_S16            = FS_SAMPLEFORMAT( 1, 16, 16, 1 ),
+
+     /* Signed 24 bit (little endian). */
+     FSSF_S24            = FS_SAMPLEFORMAT( 2, 24, 24, 1 ),
+
+     /* Signed 32 bit (native endian). */
+     FSSF_S32            = FS_SAMPLEFORMAT( 3, 32, 32, 1 ),
 } FSSampleFormat;
 
+/* Number of defined sample formats */
+#define FS_NUM_SAMPLEFORMATS  4
+
+/* These macros extract information about the sample format. */
+#define FS_SAMPLEFORMAT_INDEX( fmt )  (((fmt) & 0x0000000F)      )
+
+#define FS_BITS_PER_SAMPLE( fmt )     (((fmt) & 0x000007F0) >>  4)
+
+#define FS_BYTES_PER_SAMPLE( fmt )    (((fmt) & 0x000007F0) >>  7)
+
+#define FS_SAMPLEFORMAT_DEPTH( fmt )  (((fmt) & 0x0003f800) >> 11)
+
+#define FS_SIGNED_SAMPLEFORMAT( fmt ) (((fmt) & 0x00040000) ? 1 : 0)
+
+
 /*
  * Each buffer description flag validates one field of the buffer description.
  */
@@ -176,6 +252,39 @@ typedef enum {
      FMCAPS_SEEK       = 0x00000001,  /* supports SeekTo                */
 } FSMusicProviderCapabilities;
 
+/*
+ * Track ID.
+ */
+typedef unsigned int FSTrackID;
+
+#define FS_TRACK_DESC_ARTIST_LENGTH   32
+#define FS_TRACK_DESC_TITLE_LENGTH    125
+#define FS_TRACK_DESC_ALBUM_LENGTH    125
+#define FS_TRACK_DESC_GENRE_LENGTH    32
+#define FS_TRACK_DESC_ENCODING_LENGTH 32
+
+/*
+ * Description of a track provided by a music provider.
+ */
+typedef struct {
+     char  artist[FS_TRACK_DESC_ARTIST_LENGTH];     /* Artist */
+     char  title[FS_TRACK_DESC_TITLE_LENGTH];       /* Title */
+     char  album[FS_TRACK_DESC_ALBUM_LENGTH];       /* Album */
+     short year;                                    /* Year */
+     char  genre[FS_TRACK_DESC_GENRE_LENGTH];       /* Genre */
+     char  encoding[FS_TRACK_DESC_ENCODING_LENGTH]; /* Encoding (for example: MPEG Layer-1) */
+     int   bitrate;                                 /* Bitrate in bits/s */
+} FSTrackDescription;
+
+/*
+ * Called for each track provided by a music provider.
+ */
+typedef DFBEnumerationResult (*FSTrackCallback) (
+     FSTrackID                track_id,
+     FSTrackDescription       desc,
+     void                    *callbackdata
+);
+
 /*
  * <i><b>IFusionSound</b></i> is the main FusionSound interface. Currently it
  * can only be retrieved by calling <i>IDirectFB::GetInterface()</i>. This will
@@ -633,6 +742,11 @@ DEFINE_INTERFACE( IFusionSoundPlayback,
      );
 )
 
+/*
+ * Called after each buffer write.
+ */
+typedef void (*FMBufferCallback)( int length, void *ctx );
+
 /*
  * <i>No summary yet...</i>
  */
@@ -649,26 +763,91 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
      );
 
      /*
-      * Get a surface description that best matches the music
+      * Enumerate all tracks contained in the file.
+      *
+      * Calls the given callback for all available tracks.<br>
+      * The callback is passed the track id that can be
+      * used to select a track for playback using 
+      * IFusionSoundMusicProvider::SelectTrack().
+      */
+     DFBResult (*EnumTracks) (
+          IFusionSoundMusicProvider *thiz,
+          FSTrackCallback            callback,
+          void                      *callbackdata
+     );
+
+     /*
+      * Get the unique ID of the current track.
+      */
+     DFBResult (*GetTrackID) (
+          IFusionSoundMusicProvider *thiz,
+          FSTrackID                 *ret_track_id
+     );
+
+     /*
+      * Get a description of the current track.
+      */
+     DFBResult (*GetTrackDescription) (
+          IFusionSoundMusicProvider *thiz,
+          FSTrackDescription        *desc
+     );
+
+     /*
+      * Get a stream description that best matches the music
       * contained in the file.
       */
      DFBResult (*GetStreamDescription) (
           IFusionSoundMusicProvider *thiz,
           FSStreamDescription       *desc
      );
+     
+     /*
+      * Get a buffer description that best matches the music
+      * contained in the file.
+      */
+     DFBResult (*GetBufferDescription) (
+          IFusionSoundMusicProvider *thiz,
+          FSBufferDescription       *desc
+     );
 
    /** Playback **/
 
      /*
-      * Play music rendering it into the destination stream.
+      * Select a track by its unique ID.
       */
-     DFBResult (*PlayTo) (
+     DFBResult (*SelectTrack) (
+          IFusionSoundMusicProvider *thiz,
+          FSTrackID                  track_id 
+     );
+     
+     /*
+      * Play selected track rendering it into
+      * the destination stream.
+      */
+     DFBResult (*PlayToStream) (
           IFusionSoundMusicProvider *thiz,
           IFusionSoundStream        *destination
      );
 
      /*
-      * Stop rendering into the destination stream.
+      * Play selected track rendering it into
+      * the destination buffer.
+      *
+      * Optionally a callback can be registered
+      * that is called after each buffer write.<br>
+      * The callback is passed the number of 
+      * samples per channels actually written
+      * to the destination buffer.
+      */
+     DFBResult (*PlayToBuffer) (
+          IFusionSoundMusicProvider *thiz,
+          IFusionSoundBuffer        *destination,
+          FMBufferCallback           callback,
+          void                      *ctx
+     );
+
+     /*
+      * Stop playback.
       */
      DFBResult (*Stop) (
           IFusionSoundMusicProvider *thiz
@@ -677,7 +856,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
    /** Media Control **/
 
      /*
-      * Seeks to a position within the stream.
+      * Seeks to a position within the current track.
       */
      DFBResult (*SeekTo) (
           IFusionSoundMusicProvider *thiz,
@@ -685,7 +864,10 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
      );
 
      /*
-      * Gets current position within the stream.
+      * Gets current position within the current track.
+      *
+      * Returns <i>DFB_EOF</i> if the playback
+      * of the current track is finished.
       */
      DFBResult (*GetPos) (
           IFusionSoundMusicProvider *thiz,
@@ -693,7 +875,7 @@ DEFINE_INTERFACE(   IFusionSoundMusicProvider,
      );
 
      /*
-      * Gets the length of the stream.
+      * Gets the length of the current track.
       */
      DFBResult (*GetLength) (
           IFusionSoundMusicProvider *thiz,
